/**
 * 此js文件的设计意在满足 app内随堂测详情页 与 站外分享h5 或 cms随堂测预览
 * 根据参数判断是以上哪种场景
 * 默认是app内（不以连接的形式不需要任何参数）
 * @return {[type]} [description]
 */
(function(){
	/*============= Native 与 h5 交互 =============*/
	/*
	 * ios WebviewJavascriptBriage
	 * android newslistener / imglistener
	 */
	var SHAREMAPPER = {
		1: "wxtimeline", //"微信朋友圈"
		2: "wxsession", //"微信好友"
		3: "weibo", //"新浪微博"
		4: "qqzone" //"QQ空间"
	}
	var PageType = {
		APP: "app",
		HFIVE: "hfive",
		PREVIEW: "preview"
	}
	var OS = {
		IOS: "ios",
		ANDROID: "android"
	}
	var QuizAnswerType = {
    	QAT001: "QAT001",
    	QAT002: "QAT002"
	}
	var QuizOptionType = {
		QOT001: "QOT001",
   	 	QOT002: "QOT002",
    	QOT003: "QOT003"
	}
	var config = {
		icon: location.protocol+"//"+location.host+location.pathname.split("quiz.html")[0]+"/images/testicon.png",
		webSite: "http://www.9icaishi.com",
		type: "video/mp4",
		app: "http://a.app.qq.com/o/simple.jsp?pkgname=com.caishi.cronus"
	}

	/**
	 * 接收native传递过来的quiz信息
	 */
	var QuizData;

	var util = {
		randomString: function(len) {
	    　　len = len || 32;
	    　　var $chars = 'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678';    
	        /****默认去掉了容易混淆的字符oOLl,9gq,Vv,Uu,I1****/
	    　　var maxPos = $chars.length;
	    　　var pwd = '';
	    　　for (i = 0; i < len; i++) {
	    　　　　pwd += $chars.charAt(Math.floor(Math.random() * maxPos));
	    　　}
	    　　return pwd;
	    }
	}

	/**
	 * [app内响应字体大小调整]
	 */
	function resizeFontSize(size,response){
		switch(size){
			case 0 : $(".content").removeClass("fs-md").removeClass("fs-lg").addClass("fs-sm"); return;
			case 1 : $(".content").removeClass("fs-sm").removeClass("fs-lg").addClass("fs-md"); return;
			case 2 : $(".content").removeClass("fs-sm").removeClass("fs-md").addClass("fs-lg"); return;
			default: $(".content").removeClass("fs-sm").removeClass("fs-lg").addClass("fs-md"); return;
		}
	}
	window.resizeFontSize = resizeFontSize;

	/**
	 * 调整提交按钮 位置
	 */
	function adjustSubmitBtn(l){
		$(".btn-submit").css("left",l);
	}
	window.adjustSubmitBtn = adjustSubmitBtn;

	/**
	 * [adjustBodyWidth ios 8 以下webview加载宽度bug]
	 */
	function adjustBodyWidth(w){
		$("body").width(w);
	}
	window.adjustBodyWidth = adjustBodyWidth;

	/**
	 * [initLoadData ]
	 */
	function initLoadData(data){
		/**
		 * ios data 是json，需要解析，android data为object
		 */
		if(typeof data == "undefined"){
			return;
		}
		if(typeof data == "string"){
			QuizData = JSON.parse(data);
		}else{
			QuizData = data;
		}
		QuizData.answerIsRight = true;
		page.quizInfo = QuizData;
		page.renderDom(page.quizInfo);
	}
	window.initLoadData = initLoadData;
	/**
	 * [initKoData 客户端传递 答题状态与答题结果]
	 */
	function initKoData(data){
		/**
		 * ios data 是json，需要解析，android data为object
		 */
		if(typeof data == "undefined"){
			return;
		}
		if(typeof data == "string"){
			page.koData = JSON.parse(data);
		}else{
			page.koData = data;
		}
		page.renderKoDom(page.koData);
	}
	window.initKoData = initKoData;
	/**
	 * [setNightMode ]
	 * @param {[type]} type [0:正常模式，1:夜间模式]
	 */
	function setNightMode(type){
		if(type && type == 1){
			$("body").addClass("nightMode");
		}else{
			$("body").removeClass("nightMode");
		}
	}
	window.setNightMode = setNightMode;
	/**
	 * [connectWebViewJavascriptBridge， ios js 与 native 交互的桥梁]
	 */
	function connectWebViewJavascriptBridge(callback) {
        if (window.WebViewJavascriptBridge) { 
        	return callback(WebViewJavascriptBridge); 
        }
        if (window.WVJBCallbacks) { 
        	return window.WVJBCallbacks.push(callback); 
        }
        window.WVJBCallbacks = [callback];
        var WVJBIframe = document.createElement('iframe');
        WVJBIframe.style.display = 'none';
        WVJBIframe.src = 'wvjbscheme://__BRIDGE_LOADED__';
        document.documentElement.appendChild(WVJBIframe);
        setTimeout(function() { 
        	document.documentElement.removeChild(WVJBIframe) 
        }, 0)
	}

	function Page(){
		this.pageType = this.getPageType(); // [ app, hfive, preview ]
		this.quizAnswerType = QuizAnswerType.QAT001;
		this.contentBox = $(".quiz-content");
		this.quizInfo = {};
		this.koData;
		this.init();
	}

	Page.prototype = {
		init: function(){
			var plat = this.getUA();
			if(this.pageType == PageType.APP){
				if(plat == OS.IOS){
					this.initIOS();
				}
				this.checkMultiSelect();
				this.setOptionImgWidth();
			}
			if(this.pageType == PageType.HFIVE || this.pageType == PageType.PREVIEW){
				this.initData();
				if(this.isWechat()){
					this.initWxConfig();
				}
			}
		},
		checkMultiSelect: function(){
			var rightAnswer = $(".options-box li.right");
			if(rightAnswer.length > 1){
				$(".options-box li").data("click","0");
			}
		},
		setOptionImgWidth: function(){
			try{
				var w = 120;
				for(var i = 0; i < $(".options-box li").length; i++){
					var pw = $(".options-box li:eq("+i+")").find("img").attr("src").match(/w=(\d*)/);
		            if(pw && pw[1] && pw[1] < w){
		            	w = pw[1];
		            }
				}
				$(".options-box li img").width(w);
			}catch(e){

			}
		},
		bindEvent: function(){
			var Self = this,
				plat = Self.getUA();

			$(".options-box").on('click', 'li', function (e) {
				var isClick = $(this).data("click");
				if(isClick){
					if(isClick == "0"){
						$(this).addClass("active").data("click","1");
					}else if(isClick == "1"){
						$(this).removeClass("active").data("click","0");
					}
				}else{
					$(this).addClass("active").siblings().removeClass("active");
				}
				if(Self.quizInfo.answerStatus == 0 && Self.quizAnswerType == QuizAnswerType.QAT001){
					$('.btn-submit').click();
				}
			}); 


			$('.btn-submit').on("click",{Self: this},function(){
				document.cookie = "test=helloworld";
				if(Self.quizInfo.answerStatus == 1) return;
				var activeAnswer = $(".options-box li.active");
				var rightAnswer = $(".options-box li.right");
				if(!activeAnswer || activeAnswer.length == 0){
					if(Self.pageType == PageType.HFIVE){
						alert("未选择任何选项");
						return;
					}
				}
				
				var optionId = []
				if(activeAnswer && activeAnswer.length != 0){
					for(var i = 0; i < activeAnswer.length; i++){
						optionId.push($(activeAnswer[i]).data("id"));
						if(!$(activeAnswer[i]).hasClass("right")){
							$(activeAnswer[i]).addClass("error");
							Self.quizInfo.answerIsRight = false;
						}
					}
					if(activeAnswer.length != rightAnswer.length){
						Self.quizInfo.answerIsRight = false;
					}
					Self.quizInfo.answerStatus = 1;
					try{
						Self.quizInfo.answerIsRight ? (Self.koData.Y = parseInt(Self.koData.Y) + 1) : (Self.koData.N = parseInt(Self.koData.N) + 1);
						Self.renderKoDom(Self.koData);
					}catch(e){}
					Self.contentBox.addClass("already").removeClass("yetnot");
				}
				
				if(Self.pageType == PageType.APP){
					if(plat == OS.IOS){
						WebViewJavascriptBridge.callHandler("receiveJavaScriptHandler",{
							action: 5,
							name: "quizSubmit",
							data: {optionId: optionId}
						},function(response){
							
						})
					}else if(plat == OS.ANDROID){
						newslistner.quizSubmit(optionId.join(","));
					}else{
						return;
					}
				}else{

				}
			})

			if(this.pageType == PageType.HFIVE){
				$('.share-icon-list-app li').on("click",function(){
					var type = $(this).data("type");
					var image = "<img src='images/friendshare.png' class='pull-right' style='height: 52px;'>";
					switch(type){
						case 3: window.open('http://v.t.sina.com.cn/share/share.php?title='+encodeURIComponent(document.title)+'&url='+encodeURIComponent(location.href)+'&source=bookmark','_blank','width=450,height=400');break;
						case 4: window.open('http://sns.qzone.qq.com/cgi-bin/qzshare/cgi_qzshare_onekey?url='+encodeURIComponent(document.location.href));break;
						default: $('.Guide-modal').html(image).show(0,function(){
							var that = this;
							setTimeout(function(){
								$(that).hide();
							},3000)
						}); break;
					}
				})
				$('.btn-openApp').on("click",function(){
					window.open(config.app);
				})
				wx.ready(function(){
					var imgUrl = config.icon;
					// 分享到朋友圈
					wx.onMenuShareTimeline({
					    title: document.title + " | 唔哩", // 分享标题
					    link: location.href, // 分享链接
					    imgUrl: imgUrl, // 分享图标
					    success: function () { 
					        // 用户确认分享后执行的回调函数
					    },
					    cancel: function () { 
					        // 用户取消分享后执行的回调函数
					    }
					});
					// 分享给朋友
					wx.onMenuShareAppMessage({
					    title: document.title, // 分享标题
					    desc: document.title, // 分享描述
					    link: location.href, // 分享链接
					    imgUrl: imgUrl, // 分享图标
					    type: '', // 分享类型,music、video或link，不填默认为link
					    dataUrl: '', // 如果type是music或video，则要提供数据链接，默认为空
					    success: function () { 
					    	// alert("分享成功");
					        // 用户确认分享后执行的回调函数
					    },
					    cancel: function () { 
					    	// alert("取消分享");
					        // 用户取消分享后执行的回调函数
					    },
					    fail: function(){
					    	// alert("分享失败");
					    },
					    complete: function(){
					    	// alert("分享完成");
					    },
					    trigger: function(){
							// alert("分享触发");
					    }
					});
				})
				wx.error(function(e){
					// alert(JSON.stringify(e));
				})
			}
			document.body.addEventListener('touchstart', function (e) {}); 
			$(".content a").on('click', function (e) { e.preventDefault(); }); 
			new NoClickDelay("options-box");
		},
		initWxConfig: function(){
			var time = new Date().getTime().toString().substr(0,10);
			var str = util.randomString(16);
			$.ajax({
				type: "get",
				url: "http://cms.365liuda.com/hfive/signature",
				data: {
					noncestr: str,
                	timestamp: time,
                	url:encodeURIComponent(location.href.split("#")[0])
				},
				success: function(data){
					if(data && data != ""){
						wx.config({
						    debug: false, // 开启调试模式,调用的所有api的返回值会在客户端alert出来，若要查看传入的参数，可以在pc端打开，参数信息会通过log打出，仅在pc端时才会打印。
						    appId: 'wx8e894199803f6356', // 必填，公众号的唯一标识
						    timestamp: time, // 必填，生成签名的时间戳
						    nonceStr: str, // 必填，生成签名的随机串
						    signature: data,// 必填，签名，见附录1
						    jsApiList: ['onMenuShareTimeline','onMenuShareAppMessage'] // 必填，需要使用的JS接口列表，所有JS接口列表见附录2
						});
					}
				},
				error: function(err){
					console.log(err);
					//alert(err);
				},
				dataType: "jsonp"
			})
		},
		getUA: function(){
			var UA = navigator.userAgent.toLowerCase(), plat = null;
			if(UA.match(/iphone|ipad|ipod/)){
				plat = OS.IOS;
			}
			if(UA.match(/android/)){
				plat = OS.ANDROID;
			}
			this.osType = plat;
			return plat;
		},
		initIOS: function(){
		    connectWebViewJavascriptBridge(function(bridge) {
		        bridge.registerHandler("resizeFontSize",resizeFontSize);
		        bridge.registerHandler("adjustSubmitBtn",adjustSubmitBtn);
		        bridge.registerHandler("adjustBodyWidth",adjustBodyWidth);
		        bridge.registerHandler("initLoadData",initLoadData);
		        bridge.registerHandler("initKoData",initKoData);
		        bridge.registerHandler("setNightMode",setNightMode);
		    })
		    $("body").addClass("ios-device");
		},
		initData: function(){
			var that = this;
			$.get("/v1/quiz/detail?id="+this.quizId,function(res){
				if(res.code == "10000" && res.data){
					that.quizInfo = res.data;
					that.quizInfo.answerIsRight = true;
					that.renderDom(res.data);
				}else{
					that.d404();
				}
			})
			$.get("/v1/quiz/answer/statistic?id="+this.quizId,function(res){
				if(res.code == "10000" && res.data){
					that.koData = res.data;
					that.renderKoDom(that.koData);
				}else{
					that.d404();
				}
			})
		},
		getUrlParam: function() {
		    var search = window.location.search.substr(1);
		    var mappers = search.split("&");
		    var hash = {};
		    for(var i in mappers){
		        var index = mappers[i].indexOf("=");
		        hash[mappers[i].substring(0,index)] = mappers[i].substring(index+1)
		    }
		    return hash;
		},
		getPageType: function(){
			var type = PageType.APP;
			try{
				var param = this.getUrlParam();
				if(param.quizId){
					type = PageType.HFIVE;
					this.quizId = param.quizId;
				}
				if(param.preview){
					type = PageType.PREVIEW;
				}
			}catch(e){

			}
			return type;
		},
		renderDom: function(quiz){
			quiz.quizAnswerType && (this.quizAnswerType = quiz.quizAnswerType);
			
			var titleBox = $(".title-box");
			var optionsBox = $(".options-box");
			var answerDesc = $(".answer-box .analysis div");
			var buildDesc = function(){
				var html = "";
				quiz.description && (html += quiz.description);
				quiz.descriptionImage && (html += "<img src=\""+quiz.descriptionImage+"\">");
				return html;
			}
			var selectAnswer = quiz.selectAnswer && quiz.selectAnswer.split(",") || [];
			var buildOption = function(){
				var html = "";
				var abc = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
				var rightAnswer = [];
				try{
					html += "<ul>"
					for(var i = 0; i < quiz.quizOptionInfoList.length; i++){
						var li = "", flag = abc.charAt(i);
						li += "<li class=\"option\" data-id=\""+quiz.quizOptionInfoList[i].id+"\">";
						li +=	 "<div>";
						li += 	 	"<div class=\"option-img\">";
                        li +=	    	"<img src=\""+quiz.quizOptionInfoList[i].image+"\">";
                        li +=	 	"</div>"; 
                        li +=	 	"<div class=\"option-content\">";
		                li +=    		"<span class=\"pull-left ta-c\">";
		                li +=        		"<i class=\"radio\"></i>";
		                li +=    		"</span>";
		                li +=    		"<span class=\"pull-left\">"+flag+".</span>"; 
		                li +=    		"<div class=\"option-text\">";
	                	li += 				quiz.quizOptionInfoList[i].text;
		                li +=    		"</div>";   
		                li +=    	"</div>";
		                li +=	 "</div>";
		                li +="</li>";
		                if(quiz.quizOptionInfoList[i].answerStatus == 1){ // 这里是选项的状态 是否正确
		                	li = li.replace(/option/,"option right");
		                	rightAnswer.push(quiz.quizOptionInfoList[i].id);
		                }else{
		                	if(quiz.answerStatus == 1 && selectAnswer.indexOf(quiz.quizOptionInfoList[i].id) > -1){ // 这里是答题状态 是否答题
			                	li = li.replace(/option/,"option error");
			                	quiz.answerIsRight = false;
			                }
		                }
		                html += li;
					}
					if(selectAnswer.length && selectAnswer.length != rightAnswer.length){
						quiz.answerIsRight = false;
					}
					if(!quiz.answerIsRight){
						this.renderKoDom(this.kodata);
					}
					html += "</ul>"
				}catch(e){

				}
				return html
			}
			
			titleBox.html(buildDesc());
			if(quiz.quizOptionType == QuizOptionType.QOT002 || quiz.quizOptionType == QuizOptionType.QOT003){
				this.contentBox.addClass("has-img");
			}
			if(quiz.analysis && quiz.analysis != ""){
				this.contentBox.addClass("has-analysis");
			}
			if(quiz.answerStatus == 1){
				this.contentBox.removeClass("yetnot").addClass("already");
			}
			optionsBox.html(buildOption.call(this));
			answerDesc.html(quiz.analysis);

			if(quiz.quizAnswerType == QuizAnswerType.QAT002){
				this.contentBox.addClass("multiple");
				optionsBox.find("li").data("click","0");
			}
			/**
			 * 设置导航栏title
			 */
			document.title = quiz.title;
			/**
			 * 显示底部banner
			 */
			if(this.pageType == PageType.HFIVE){
				$(".foot-download").addClass("in");
				this.contentBox.addClass("hfive");
				$(".banner-img").attr("src",quiz.cover).show();
			}
			/*ios 设置 选项 背景色为eeeeee*/
			if(this.osType == OS.IOS){
				$(".options-box li option-content").css("background","#eee");
			}
			/**
			 * bind event
			 */
			this.bindEvent();
		},
		renderKoDom: function(koData){
			if(!koData) return;
			var koBox = $(".ko-box");
			var dy = parseInt(koData.Y || 30);
			var dn = parseInt(koData.N || 70);
			var total = dy + dn;
			var y = Math.round(dy / total * 100) + "%";
			var n = Math.round(dn / total * 100) + "%";
			var html = "";
			html +=    "<div class=\"kodata\">";
            html +=    "<dl class=\"clearfix\">";
            html +=    "<dt><i class=\"radio\"></i></dt>";
            html +=    "<dd>";
            html +=	   "<div class=\"progress-box\">";
            html +=    "<span class=\"count\" style=\"left: {{y}}\">{{y}}</span>";
            html +=    "<span class=\"progress\" style=\"width: {{y}}\"></span>";
            html +=    "</div>";
        	html +=    "</dd>";
            html +=    "</dl>";
            html +=    "<dl class=\"clearfix\">";
            html +=    "<dt><i class=\"radio\"></i></dt>";
            html +=    "<dd>";
         	html +=    "<div class=\"progress-box\">";
            html +=    "<span class=\"count\"  style=\"left: {{n}}\">{{n}}</span>";
            html +=    "<span class=\"progress\" style=\"width: {{n}};\"></span>";
            html +=    "</div>";
            html +=    "</dd>";
            html +=    "</dl>";
        	html +=    "</div>";
            html +=    this.quizInfo.answerIsRight ? "<b class=\"kotip\"> 恭喜你，你打败了{{n}}的人 </b>" : "<b class=\"kotip\"> 再接再厉，重在掺和 </b>";
            html = html.replace(/{{y}}/g,y);
            html = html.replace(/{{n}}/g,n);
            koBox.html(html);
		},
		isWechat: function(){
			if(navigator.userAgent.match(/MicroMessenger/i)){
				return true;
			}else{
				return false;
			}
		},
		d404: function(){
			location.href = "404.html"
		}
	}
	var page = new Page();
	function NoClickDelay(el) {
	    this.element = typeof el == 'object' ? el: document.getElementById(el);       
	    if (window.Touch)  this.element.addEventListener('touchstart', this, false);
	}
	NoClickDelay.prototype = {
	    handleEvent: function(e) {               
	        switch (e.type) {                       
	        case 'touchstart':
	            this.onTouchStart(e);
	            break;                       
	        case 'touchmove':
	            this.onTouchMove(e);
	            break;                       
	        case 'touchend':
	            this.onTouchEnd(e);
	            break;               
	        }       
	    },
	    onTouchStart: function(e) {
	        this.moved = false;
	        var deep = 5;
	        this.theTarget = document.elementFromPoint(e.targetTouches[0].clientX, e.targetTouches[0].clientY);     
	        while(deep){
	        	if (this.theTarget.nodeName == "LI") {
					this.theTarget.className += ' pressed';
	        		this.element.addEventListener('touchmove', this, false);
	        		this.element.addEventListener('touchend', this, false);       
	        	}else{
	        		this.theTarget = this.theTarget.parentNode;
	        	}
	        	deep--;
	        }          
	    },
	    onTouchMove: function(e) {
	        this.moved = true;
	        this.theTarget.className = this.theTarget.className.replace(/ ?pressed/gi, '');       
	    },
	    onTouchEnd: function(e) {
	        this.element.removeEventListener('touchmove', this, false);
	        this.element.removeEventListener('touchend', this, false);                     
	        if (!this.moved && this.theTarget) {
	            this.theTarget.className = this.theTarget.className.replace(/ ?pressed/gi, '');                        
	            // var theEvent = document.createEvent('MouseEvents');
	            // theEvent.initEvent('click', true, true);
	            // this.theTarget.dispatchEvent(theEvent);                
	        }
	        this.theTarget = undefined;       
	    }
	};
}())
